#include "stdafx.h"
#include "GrfxWindowD3D.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif


CGrfxWindowD3D::CGrfxWindowD3D()
: m_pD3D(NULL)
, m_pD3DDevice(NULL)
, m_camera_x(0)
, m_camera_y(10)
, m_camera_z(-25)
, m_lookat_x(0)
, m_lookat_y(0)
, m_lookat_z(0)
, m_upvec_z(0)
, m_rot(0)
, m_PtOn(FALSE)
, m_DirOn(FALSE)
, m_SpotOn(FALSE)
{
  ResetCameraParameters();
}

//The connections to Direct3D should have already been destroyed in the
//OnDestroy function, but just to be paranoid make sure it is destroyed here
CGrfxWindowD3D::~CGrfxWindowD3D()
{
  //destroy the graphics device
  if(m_pD3DDevice)
  {
    m_pD3DDevice->Release();
    m_pD3DDevice = NULL;
  }
  // destroy the connection to D3D
  if(m_pD3D)
  {
    m_pD3D->Release();
    m_pD3D = NULL;
  }
}

bool CGrfxWindowD3D::ReplaceDialogControl(HWND hDialogHandle, int id)
{
  //check for valid input parameters
  CWnd* pDialog = CWnd::FromHandle(hDialogHandle);
  ASSERT(pDialog!=NULL);
  CWnd* pCurrentCtrl = pDialog->GetDlgItem(id);
  if(NULL==pCurrentCtrl)
    return false;

  //Copy the contol's styles. This gives us some way to define things like border style
  //in the resource editor
  DWORD style = pCurrentCtrl->GetStyle();
  DWORD styleEx = pCurrentCtrl->GetExStyle();
  // use the control to figure out how large is the initial on-screen area for the graphics window
  CRect rect;
  pCurrentCtrl->GetWindowRect(&rect);

  // the static was just a place holder, destroy it
  pCurrentCtrl->DestroyWindow();

  // convert the screen rectangle to the dialog's client coordinates
  pDialog->ScreenToClient(&rect);

  BOOL success = CreateEx(styleEx,
                          NULL,     //CWnd default
                          NULL,     //has no name
      	                  style|WS_CHILD|WS_CLIPSIBLINGS|WS_CLIPCHILDREN|WS_VISIBLE,
	                        rect,     // Dimension and location of the place-holder
	                        pDialog,  //this is the parent
	                        id);      //Use the ID of the just destroyed static

  BringWindowToTop();

  // if we failed to create a window, display a message box to help in debugging
  if(!success)
    ::MessageBox( hDialogHandle,
                  L"CGrfxWindowD3D creation failed!",
                  L"CGrfxWindowD3D::ReplaceDialogControl",
                  MB_OK|MB_ICONSTOP );

  return (success==TRUE);
}

void CGrfxWindowD3D::ResetCameraParameters()
{
  m_camera_x = 0; m_camera_y = 10; m_camera_z = -25;
  m_lookat_x = m_lookat_y = m_lookat_z = 0;
  m_upvec_z = 0;
  m_rot = 0;
}


void CGrfxWindowD3D::SetCameraPosition( float x, float y, float z )
{
  m_camera_x = x;
  m_camera_y = y;
  m_camera_z = z;
}
void CGrfxWindowD3D::GetCameraPosition( float& x, float& y, float& z ) const
{
  x = m_camera_x;
  y = m_camera_y;
  z = m_camera_z;
}

void CGrfxWindowD3D::SetCameraLookAt( float x, float y, float z )
{
  m_lookat_x = x;
  m_lookat_y = y;
  m_lookat_z = z;
}
void CGrfxWindowD3D::GetCameraLookAt( float& x, float& y, float& z ) const
{
  x = m_lookat_x;
  y = m_lookat_y;
  z = m_lookat_z;
}

void CGrfxWindowD3D::SetCameraUpVector(float rot, float z)
{
    m_rot = rot;
    m_upvec_z = z;
}

void CGrfxWindowD3D::GetCameraUpVector(float &rot, float &z)
{
    rot = m_rot;
    z = m_upvec_z;
}

// MFC Message map macro for mapping windows messages to class functions
BEGIN_MESSAGE_MAP(CGrfxWindowD3D, CWnd)
  ON_WM_ERASEBKGND()
  ON_WM_PAINT()
  ON_WM_CREATE()
  ON_WM_DESTROY()
END_MESSAGE_MAP()

int CGrfxWindowD3D::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
  // Call the base class version first
  if (CWnd::OnCreate(lpCreateStruct) == -1)
    return -1;

  // Attempt to connect to Direct3D and create a device appropriate for this window

  // Initialize Graphics
  if(NULL == m_pD3D)
    m_pD3D = Direct3DCreate9( D3D_SDK_VERSION);

  if(NULL == m_pD3D)
    return -1;


  // Set up the structure used to create the D3dDevice.
  // Init the entire D3DPRESENT structure to 0
  D3DPRESENT_PARAMETERS present_params;
  ZeroMemory( &present_params, sizeof(present_params) );

  // Set Windowed to TRUE, since we want to do D3d in a window
  present_params.Windowed = TRUE;

  // set SwapEffect to "discard", which is the most efficient method 
  // of presenting the back buffer to the display.
  present_params.SwapEffect = D3DSWAPEFFECT_DISCARD;

  // request a back buffer format that matches the current desktop display format
  D3DDISPLAYMODE display_mode;
  m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &display_mode );
  present_params.BackBufferFormat = display_mode.Format;
  present_params.EnableAutoDepthStencil = TRUE;
  present_params.AutoDepthStencilFormat = D3DFMT_D24S8;
  present_params.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

  // Create the Direct3D device. Here we are using the default adapter (most
  // systems only have one, unless they have multiple graphics hardware cards
  // installed) and requesting the HAL (which is saying we want the hardware
  // device rather than a software one). Software vertex processing is 
  // specified since we know it will work on all cards. On cards that support 
  // hardware vertex processing, though, we would see a big performance gain 
  // by specifying hardware vertex processing.
  if( FAILED(m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd,
                                   D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                   &present_params, &m_pD3DDevice ) ) )
  {
    MessageBox(L"D3D Creation Failed during CreateDevice",L"D3DRenderer::Initialize",MB_OK);
    m_pD3D->Release();
    m_pD3D = NULL;
    return -1;
  }

  return 0;
}

void CGrfxWindowD3D::OnDestroy()
{
  // When the window is destroyed, we no longer need a connection to D3D.
  // Release all of the current connections.

  //destroy the graphics device
  if(m_pD3DDevice)
  {
    m_pD3DDevice->Release();
    m_pD3DDevice = NULL;
  }

  // destroy the abstract graphics device
  if(m_pD3D)
  {
    m_pD3D->Release();
    m_pD3D = NULL;
  }

  CWnd::OnDestroy();
}

// This just overrides the default window behavior of erasing the background of window. If we
// don't override this behavior, the window will procude a "blinking" effect.
BOOL CGrfxWindowD3D::OnEraseBkgnd(CDC* pDC)
{
  return TRUE;
}

////////////// DRAWING ///////////////////
static bool DrawLine(LPDIRECT3DDEVICE9 pDevice,
                     DWORD color,
                     D3DXVECTOR3 start,
                     D3DXVECTOR3 end)
{
  //Direct3D likes to recieve arrays of structs with vertex information. For simple, unlit
  //drawing; a vertex and color at the vertex will suffice.
  struct LineVertexFormat
  {
    D3DXVECTOR3 m_point;  // vertex location
    DWORD m_color;        // color at the vertex
  };

  if (NULL == pDevice)
    return false;

  LineVertexFormat draw_points[2];
  draw_points[0].m_point = start;
  draw_points[0].m_color = color;
  draw_points[1].m_point = end;
  draw_points[1].m_color = color;

  pDevice->SetFVF(D3DFVF_XYZ | D3DFVF_DIFFUSE);
  pDevice->DrawPrimitiveUP(D3DPT_LINELIST, 1, (CONST void *)(&draw_points), sizeof(LineVertexFormat));
  return true;
}

//To keep things as simple as possible all of the geometry is just drawn in this function.
//Later, we will abstract geometry in separate classes that are stored as instances in the
//model. But... for now, let's just draw two rectangles.
void CGrfxWindowD3D::OnPaint()
{
  CPaintDC dc(this); // device context for painting

  // Get the current size of this window
  CRect rect;
  GetClientRect(&rect);
  float width = (float)rect.Width();
  float height = (float)rect.Height();

  if(m_pD3DDevice && width > 0 && height > 0)
  {
    //perform all of our drawing
    m_pD3DDevice->BeginScene();  

    m_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
    m_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
    
    
    /* No need to set RenderState on lighting/material computation!!

    m_pD3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);

    m_pD3DDevice->SetRenderState(D3DRS_SPECULARENABLE, TRUE);
    m_pD3DDevice->SetRenderState( D3DRS_LOCALVIEWER, TRUE);
    m_pD3DDevice->SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE, D3DMCS_MATERIAL);
    m_pD3DDevice->SetRenderState(D3DRS_SPECULARMATERIALSOURCE, D3DMCS_MATERIAL);
    m_pD3DDevice->SetRenderState(D3DRS_AMBIENTMATERIALSOURCE, D3DMCS_MATERIAL);
    m_pD3DDevice->SetRenderState(D3DRS_EMISSIVEMATERIALSOURCE, D3DMCS_MATERIAL);
    m_pD3DDevice->SetRenderState(D3DRS_NORMALIZENORMALS, TRUE);
    */


    // THIS IS HOW WE SET UP THE MATRICES IN 2D
    // Set the model matrix to identity
    D3DXMATRIX identity;
    D3DXMatrixIdentity(&identity);
    m_pD3DDevice->SetTransform( D3DTS_WORLD, &identity);
    m_pD3DDevice->SetTransform( D3DTS_VIEW, &identity);
    m_pD3DDevice->SetTransform( D3DTS_PROJECTION, &identity);
/*
    D3DXMATRIX world2ndc;
    D3DXVECTOR3 scale(2.0f/width, 2.0f/height, 1.0f);
    D3DXVECTOR3 translate(-1.0f, -1.0f, 0.0);
    D3DXMatrixTransformation(&world2ndc, 
                            NULL,   // Scale reference
                            NULL,   // Scale rotation
                            &scale, // the actual Scales
                            NULL,   // Axis of rotation
                            NULL,   // The rotation
                            &translate);
    m_pD3DDevice->SetTransform(D3DTS_VIEW, &world2ndc);
*/



/* for Pixel/Vertex shaders ...
   notice, in the following code, we _DO NOT_ set the transforms!! */
    
    // Set the model matrix to identity
    D3DXMATRIX matWorld;
    D3DXMatrixIdentity(&matWorld);
    //m_pD3DDevice->SetTransform( D3DTS_WORLD, &matWorld);  <-- NOT SET!!

    // NOW IN 3D WE CONSIDER Z AXIS AND PERSPECTIVE TRANSFORM
    // Set up the camera position and projection information
    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here, we set the
    // eye five units back along the z-axis and up three units, look at the 
    // origin, and define "up" to be in the y-direction.
    D3DXMATRIX matView;
    D3DXVECTOR3 camera_pos(m_camera_x, m_camera_y, m_camera_z);
    D3DXVECTOR3 target_pos(m_lookat_x, m_lookat_y, m_lookat_z);
    D3DXVECTOR3 up_vector;
    up_vector.x = sinf( UWB_ToRadians(m_rot) );
    up_vector.y = cosf( UWB_ToRadians(m_rot) );
    up_vector.z = m_upvec_z;
    D3DXMatrixLookAtRH( &matView, &camera_pos, &target_pos, &up_vector );
    // m_pD3DDevice->SetTransform( D3DTS_VIEW, &matView );  <-- NOT SET!!

    // For the projection matrix, we set up a perspective transform (which
    // transforms geometry from 3D view space to NDC, with
    // a perspective divide making objects smaller in the distance). To build
    // a perpsective transform, we need the field of view (1/4 pi is common),
    // the aspect ratio, and the near and far clipping planes (which define at
    // what distances geometry should be no longer be rendered).
    D3DXMATRIX matProj;
    float fov = 3.14159265f/4.0f;   // field of view
    float aspect = width / height;
    float near_plane = 1.0f;
    float far_plane = 100.0f;
    D3DXMatrixPerspectiveFovRH( &matProj, fov, aspect, near_plane, far_plane);
    // m_pD3DDevice->SetTransform( D3DTS_PROJECTION, &matProj ); <-- NOT SET!!



    /* NEW CODE FOR VERTEX/PIXEL Shading: instead of defining/settnig Light/Material
       we will load effect file and set the appropriate variables!!
     */
    LPD3DXBUFFER errorBuf;
    ID3DXEffect *effect;
    DWORD dwShaderFlags = 0L;

#if defined( DEBUG ) || defined( _DEBUG )
    // Set the D3DXSHADER_DEBUG flag to embed debug information in the shaders.
    // Setting this flag improves the shader debugging experience, but still allows 
    // the shaders to be optimized and to run exactly the way they will run in 
    // the release configuration of this program.
    dwShaderFlags |= D3DXSHADER_DEBUG;
#endif
     
    HRESULT stat = D3DXCreateEffectFromFile(m_pD3DDevice, 
                       // L"Effects/1.BareBone.fx",
                       L"Effects/2.NdotL.fx",
                       NULL, NULL, dwShaderFlags, NULL, &effect, &errorBuf);
    if (D3D_OK != stat) { 
        wchar_t *msg = NULL;
        if (NULL != errorBuf) {
            msg = (wchar_t *) errorBuf->GetBufferPointer();
        }
       MessageBox(L"Effect Loading Failed!", msg, MB_OK);
       return;
    }

    
    //clear the draw buffer to light gray
    D3DCOLOR background = D3DCOLOR_COLORVALUE(0.8f, 0.8f, 0.8f, 0);
    m_pD3DDevice->Clear(0,                // number of rectangles, 0 means the entire area
                        NULL,             // rectangles to be cleared, NULL means the entire area
                        D3DCLEAR_TARGET|  // clear target (color)
                        D3DCLEAR_ZBUFFER,
                        background,       // new color to clear to
                        1.0f,             // new Z values to clear to
                        0);               // new Stencil buffer values to clear to

    D3DXMATRIX worldI, worldIT; // inverse and inverset transpose
    D3DXMatrixInverse(&worldI, NULL, &matWorld); // in this case since world is I4, inverse is not really needed
    D3DXMatrixTranspose(&worldIT, &worldI); // inverse transpose of world! for transforming vectors!

    // Now the effect is loaded, we must set its variables (in the rendering pipe)
    effect->SetMatrix("world", &matWorld);
    effect->SetMatrix("view", &matView);
    effect->SetMatrix("projection", &matProj);
    effect->SetMatrix("worldIT", &worldIT);

    // can either name the rendering techniques explicitly or ...
        // effect->SetTechnique("SimpleShader");
        // effect->SetTechnique("SimpleNdotL");
    // find the first available rendering technique ...
    D3DXHANDLE tech;
    effect->FindNextValidTechnique(NULL, &tech);
    effect->SetTechnique(tech);


#define SOFT 0.4f
#define STRONG 1.0f

    /* for VERTEX/PIXEL shader ... Instead of setting RenderingPipe's light content ... 

    // Now switch on the lights ...
    D3DLIGHT9 ptLight, spotLight, dirLight;
    ptLight.Type = D3DLIGHT_POINT;
    ptLight.Position.x = -4.0f; ptLight.Position.y = 5.0f;    ptLight.Position.z = -4.0f; 
    ptLight.Direction.x = 0.0f; ptLight.Direction.y = -1.0f;  ptLight.Direction.z = 0.0f; 
    ptLight.Ambient.r  = SOFT; ptLight.Ambient.g = SOFT;   ptLight.Ambient.b = SOFT;   ptLight.Ambient.a = 0.0f;
    ptLight.Diffuse.r  = SOFT; ptLight.Diffuse.g = STRONG; ptLight.Diffuse.b = SOFT;   ptLight.Diffuse.a = 0.0f;
    ptLight.Specular.r = SOFT; ptLight.Specular.g = SOFT;   ptLight.Specular.b = SOFT; ptLight.Specular.a = 0.0f;
    ptLight.Range = 100.0f;
    ptLight.Attenuation0 = 1.0f;
    ptLight.Attenuation1 = 0.0f;
    ptLight.Attenuation2 = 0.0f;
    ptLight.Theta = 0.0f;
    ptLight.Phi = 0.0f;
    ptLight.Falloff = 0.0f;
    m_pD3DDevice->SetLight(0, &ptLight);
    m_pD3DDevice->LightEnable(0, m_PtOn);

    spotLight.Type = D3DLIGHT_SPOT;
    spotLight.Position = D3DXVECTOR3(-4.0f, 5.0f, 4.0f);
    spotLight.Direction = D3DXVECTOR3(0.0f, -1.0f, 0.0f);
    spotLight.Ambient.r  = SOFT; spotLight.Ambient.g = SOFT;   spotLight.Ambient.b = SOFT;     spotLight.Ambient.a = 0.0f;
    spotLight.Diffuse.r  = SOFT; spotLight.Diffuse.g = SOFT;   spotLight.Diffuse.b = STRONG;   spotLight.Diffuse.a = 0.0f;
    spotLight.Specular.r = SOFT; spotLight.Specular.g = SOFT;  spotLight.Specular.b = SOFT;    spotLight.Specular.a = 0.0f;
    spotLight.Range = 100.0f;
    spotLight.Attenuation0 = 1.0f;
    spotLight.Attenuation1 = 0.0f;
    spotLight.Attenuation2 = 0.0f;
    spotLight.Theta = UWB_PI/2.0f;
    spotLight.Phi = UWB_PI/1.8f;
    spotLight.Falloff = 2.0f;
    m_pD3DDevice->SetLight(1, &spotLight);
    m_pD3DDevice->LightEnable(1, m_SpotOn);

    dirLight.Type = D3DLIGHT_DIRECTIONAL;
    dirLight.Position = D3DXVECTOR3(0.0f, 5.0f, 0.0f);
    dirLight.Direction = D3DXVECTOR3(0.0f, -1.0f, 0.0f);
    dirLight.Ambient.r  = SOFT;   dirLight.Ambient.g = SOFT;   dirLight.Ambient.b = STRONG;   dirLight.Ambient.a = 0.0f;
    dirLight.Diffuse.r  = STRONG; dirLight.Diffuse.g = STRONG;   dirLight.Diffuse.b = STRONG;   dirLight.Diffuse.a = 0.0f;
    dirLight.Specular.r = SOFT;   dirLight.Specular.g = SOFT;  dirLight.Specular.b = SOFT; dirLight.Specular.a = 0.0f;
    dirLight.Range = 100.0f;
    dirLight.Attenuation0 = 1.0f;
    dirLight.Attenuation1 = 0.0f;
    dirLight.Attenuation2 = 0.0f;
    dirLight.Theta = 0.0f;
    dirLight.Phi = 0.0f;
    dirLight.Falloff = 0.0f;
    m_pD3DDevice->SetLight(2, &dirLight);
    m_pD3DDevice->LightEnable(2, m_DirOn); */

        //
        // NOTICE: these are undefined for 1.BareBone.fx shaders!!
        // Notice, the lack of error message!
        // these are only defined in 2.NdotL.fx
        // notice, the size and data fields MUST BE IDENTICAL!!
        struct {
            float lgtDiffuse[4];
            float lgtPosition[3];
        } vsLight = {
               {STRONG, STRONG, STRONG, STRONG},  // lgtDiffuse
               {0.0f, 6.0f, 0.0f}};            // lgtPosition
        effect->SetValue("myLight", &vsLight, sizeof(vsLight));

       //This MUST BE identical to the VertexShaderInput!!
      struct DeviceVertexFormat
      {
        D3DXVECTOR3 m_point;  // vertex location
        D3DXVECTOR3 m_normal;
        FLOAT m_uv[2];        // UV coordinate at the vertex
      };
      m_pD3DDevice->SetFVF(D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1);

      //Draw the two rectangles
      DeviceVertexFormat v[4];
      D3DXVECTOR3 n(0.0f, 1.0f, 0.0f);
      for(int i=0; i<4; i++)
        ZeroMemory(&(v[i]), sizeof(DeviceVertexFormat));

      v[0].m_uv[0] = 0.0f; v[0].m_uv[1] = 0.0f;
      v[1].m_uv[0] = 1.0f; v[1].m_uv[1] = 0.0f;
      v[2].m_uv[0] = 1.0f; v[2].m_uv[1] = 1.0f;
      v[3].m_uv[0] = 0.0f; v[3].m_uv[1] = 1.0f;
      v[0].m_normal = D3DXVECTOR3(-1.0f, 0.0f, 1.0f);
      v[1].m_normal = D3DXVECTOR3(1.0f, 0.0f, 1.0f);
      v[2].m_normal = D3DXVECTOR3(1.0f, 0.0f, -1.0f);
      v[3].m_normal = D3DXVECTOR3(-1.0f, 0.0f, -1.0f);
      for (int i=0; i<4; i++)
          D3DXVec3Normalize(&(v[i].m_normal), &(v[i].m_normal));


               /* VERTEX/PIXEL shader: do not need to set the material ...
                  D3DMATERIAL9 largeMaterial;
                  largeMaterial.Ambient.r = SOFT; largeMaterial.Ambient.g = SOFT; largeMaterial.Ambient.b = SOFT; largeMaterial.Ambient.a = 1.0f;
                    ...                 
                 m_pD3DDevice->SetMaterial(&largeMaterial);
                  */
           // set parameter before begin rendering!
           struct {
               float matDiffuse[4];
               float matEmmissive[4];
           } vsMatLarge = {
               {STRONG, SOFT, SOFT, SOFT}, // matDiffuse
               {SOFT, SOFT, SOFT, SOFT}};
           effect->SetValue("myMaterial", &vsMatLarge, sizeof(vsMatLarge));
           effect->SetBool("textureEnable", false);
           effect->SetTexture("textureMap", NULL);
           
    // NOW, begin rendering!
    UINT numPasses;
    effect->Begin(&numPasses, 0);
    for (UINT i = 0; i<numPasses; i++) {
        effect->BeginPass(i);
                    // Large Square
              v[0].m_point = D3DXVECTOR3(-4,0,4); 
              v[1].m_point = D3DXVECTOR3(4,0,4);  
              v[2].m_point = D3DXVECTOR3(4,0,-4); 
              v[3].m_point = D3DXVECTOR3(-4,0,-4);
              m_pD3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, (CONST void *)v, sizeof(DeviceVertexFormat));
      

           // now get ready to render the small square
               /* NOT Needed for VERTEX/PIXEL shading ..   
                  D3DMATERIAL9 smallMaterial;
                  smallMaterial.Ambient.r = SOFT; smallMaterial.Ambient.g = SOFT; smallMaterial.Ambient.b = SOFT; smallMaterial.Ambient.a = 1.0f;
                  ...
                  m_pD3DDevice->SetMaterial(&smallMaterial);
                */

          struct {
               float matDiffuse[4];
               float matEmmissive[4];
           } vsMatSmall = {
               {SOFT, SOFT, STRONG, SOFT}, // matDiffuse
               {SOFT, SOFT, SOFT, SOFT}};
           effect->SetValue("myMaterial", &vsMatSmall, sizeof(vsMatSmall));
           // Texture ...
           LPDIRECT3DTEXTURE9 texture;
           D3DXCreateTextureFromFile(m_pD3DDevice, L"Textures/Building1.jpg", &texture);
           effect->SetBool("textureEnable", true);
           effect->SetTexture("textureMap", texture);
           effect->CommitChanges();  // changing parameters during a rendering pass is pretty expensive!          
    
              v[0].m_point = D3DXVECTOR3(-9,0,9);
              v[1].m_point = D3DXVECTOR3(-4,0,9);
              v[2].m_point = D3DXVECTOR3(-4,0,4);
              v[3].m_point = D3DXVECTOR3(-9,0,4);
              m_pD3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, (CONST void *)v, sizeof(DeviceVertexFormat));     

        effect->EndPass();
    } // effect's passes
    effect->End();

           // Now, set up for using pre-defined rendering pipe ...
           // still need to set the transforms for regular rendering!!
           m_pD3DDevice->SetTransform(D3DTS_WORLD, &matWorld);
           m_pD3DDevice->SetTransform(D3DTS_VIEW, &matView);
           m_pD3DDevice->SetTransform(D3DTS_PROJECTION, &matProj);
          #define kDelta 0.05f
           //Draw X Axis
          DWORD red = D3DCOLOR_COLORVALUE(1, 0, 0, 1);
          DrawLine( m_pD3DDevice, red, D3DXVECTOR3(kDelta, kDelta, kDelta), D3DXVECTOR3(6+kDelta, kDelta, kDelta) );
          //Draw Y Axis
          DWORD green = D3DCOLOR_COLORVALUE(0, 1, 0, 1);
          DrawLine( m_pD3DDevice, green, D3DXVECTOR3(kDelta, kDelta, kDelta), D3DXVECTOR3(kDelta, 6+kDelta, kDelta) );
          //Draw Z Axis
          DWORD blue = D3DCOLOR_COLORVALUE(0, 0, 1, 1);
          DrawLine( m_pD3DDevice, blue, D3DXVECTOR3(kDelta, kDelta, kDelta), D3DXVECTOR3(kDelta, kDelta, 6+kDelta) );
          

    m_pD3DDevice->EndScene();
    //show what we drew
    m_pD3DDevice->Present(NULL, NULL, NULL, NULL);
  }
}

BOOL CGrfxWindowD3D::PreCreateWindow(CREATESTRUCT& cs)
{
  cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS | CS_OWNDC
                                     , LoadCursor(NULL, IDC_ARROW)
                                     , NULL
                                     , NULL);
  cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  return __super::PreCreateWindow(cs);
}
