// UW Bothell Graphics Library Copyright (c) 2006 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once

/// A window control that displays Direct3D graphics. The class is
/// designed to be placed on a dialog by replacing a "placeholder"
/// type control on the dialog.
/// \see MSDN documentation on the MFC CWnd class
///      http://msdn2.microsoft.com/en-us/library/1xb05f0h(VS.80).aspx
class CGrfxWindowD3D : public CWnd
{
public:
  CGrfxWindowD3D();
  virtual ~CGrfxWindowD3D();

  /// This class is designed to be placed on a window's dialog by creating a "placerholder"
  /// control in the dialog editor and then placing this window in the location of the placeholder.
  /// This is done by calling ReplaceDialogControl in the dialog's OnInitDialog routine.
  /// Example:
  ///   BOOL CTutorialDlg::OnInitDialog()
  ///   {
  ///     CDialog::OnInitDialog();
  ///     m_GrfxWnd.ReplaceDialogControl(m_hWnd, IDC_GRFX_PLACEHOLDER);
  ///     return TRUE;  // return TRUE  unless you set the focus to a control
  ///   }
  ///
  /// \param  hDialogHandle handle to the parent dialog
  /// \param  id ID for the placeholder control that was set on the dialog in the dialog editor
  /// \return true on successful replacement
  bool ReplaceDialogControl(HWND hDialogHandle, int id);

  ///
  /// Set all of the camera parameters to some default values
  void ResetCameraParameters();

  void SetCameraPosition( float x, float y, float z );
  void GetCameraPosition( float& x, float& y, float& z) const;

  void SetCameraLookAt( float x, float y, float z );
  void GetCameraLookAt( float& x, float& y, float& z ) const;

  void SetCameraUpVector(float rot, float z);
  void GetCameraUpVector(float &rot, float &z);

  void PointOn(BOOL on) { m_PtOn = on; }
  void DirOn(BOOL on) { m_DirOn = on; }
  void SpotOn(BOOL on) { m_SpotOn = on; }

protected:
  /// A message map is an MFC macro for mapping window's events (paint, size, mouse,...) that
  /// occur on a window to functions in this class.
  /// The functions in this class that begin with afx_msg are "handlers" for window's messages
  DECLARE_MESSAGE_MAP()

  /// Called when this window recieves a WM_CREATE message (event)
  /// Where all of the connections to Direct3D should be made
  /// From MSDN docs.
  ///   Return 0 to continue creation of window
  ///   Return -1 to destroy the window
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);

  /// Called when this window receives a WM_DESTROY message (event)
  /// Connections to Direct3D should be release when the window is destroyed
  afx_msg void OnDestroy();

  /// Called when this window receives a WM_ERASEBKGND message (event)
  /// Typical windows fill the background with some brush, but when writing
  /// graphics windows that draw everything at once, there is no need to clear
  /// the background (which would cause a flicker effect.) This function always
  /// just returns without clearing the background
  afx_msg BOOL OnEraseBkgnd(CDC* pDC);

  /// Called when this window receives a WM_PAINT message (event)
  /// Windows has notified this window that it needs to be redrawn.
  /// This is where all of the graphical drawing takes place.
  afx_msg void OnPaint();

protected:
  /// Called just prior to the actual window creation. This class represents a window
  /// and hooks to a HWND handle that Win32 creates. Prior to the window creation, this
  /// class gets an opportunity to step in and modify the style flags used by the Win32
  /// CreateWindowEx(...) function.
  /// OpenGL windows require a window's style of CS_OWNDC. This setting is also
  /// fine for DirectX windows.
  /// \param cs modify this structure to modify the window creation style flags
  /// \return TRUE if the window should be created
  virtual BOOL PreCreateWindow(CREATESTRUCT& cs);

protected:
  ///
  /// Abstract connection to the Direct3D graphics system.
  LPDIRECT3D9       m_pD3D;
  ///
  /// Direct3D connection specific to this window
  LPDIRECT3DDEVICE9 m_pD3DDevice;

  BOOL m_PtOn, m_SpotOn, m_DirOn;

  float m_camera_x, m_camera_y, m_camera_z; // camera position
  float m_lookat_x, m_lookat_y, m_lookat_z; // camera look at
  float m_upvec_x, m_upvec_y, m_upvec_z;    // the upvector (this is the m_rot result)
  float m_rot;  // degree rotation of up vector
};
