#include "uwbgl_D3DDrawHelper9.h"
#include "uwbgl_D3DGraphicsSystem7.h"
#include <math.h>

UWBD3D_DrawHelper::UWBD3D_DrawHelper()
: m_pMatrixStack(NULL)
{
  // Do not initialize stack in the constructor. Stack will be initialized
  // the first time it is actually needed
}

UWBD3D_DrawHelper::~UWBD3D_DrawHelper()
{
  if(m_pMatrixStack)
  {
    m_pMatrixStack->Release();
    m_pMatrixStack = NULL;
  }
}

bool UWBD3D_DrawHelper::DrawPoint(vec3 position)
{
  //Direct3D likes to recieve arrays of structs with vertex information. For simple, unlit
  //drawing; a vertex and color at the vertex will suffice.
  struct PointVertexFormat
  {
    D3DXVECTOR3 m_point;  // vertex location
    D3DXVECTOR3 m_normal;
    float m_pointsize;    // point size
    DWORD m_color;        // color at the vertex
  };

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if (NULL == pDevice)
    return false;

  PointVertexFormat draw_point;
  draw_point.m_point.x = position.x;
  draw_point.m_point.y = position.y;
  draw_point.m_point.z = position.z;
  draw_point.m_normal = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
  draw_point.m_pointsize = m_PointSize;
  draw_point.m_color = m_Color1.ARGB();

  pDevice->SetRenderState(D3DRS_POINTSPRITEENABLE, TRUE);  // enable points to be texture mapped

  pDevice->SetFVF(D3DFVF_XYZ | D3DFVF_NORMAL|D3DFVF_PSIZE | D3DFVF_DIFFUSE);
  pDevice->DrawPrimitiveUP(D3DPT_POINTLIST, 1, (CONST void *)(&draw_point), sizeof(PointVertexFormat));
  return true;
}

bool UWBD3D_DrawHelper::DrawLine(vec3 start, vec3 end)
{
  //Direct3D likes to recieve arrays of structs with vertex information. For simple, unlit
  //drawing; a vertex and color at the vertex will suffice.
  struct LineVertexFormat
  {
    D3DXVECTOR3 m_point;  // vertex location
    D3DXVECTOR3 m_normal;
    DWORD m_color;        // color at the vertex
    FLOAT m_u0, m_v0;     // stage 0 texture coordinates
  };

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if (NULL == pDevice)
    return false;

  vec3 d = end - start;
  D3DXVECTOR3 n;
  if ((d.x > 0.01f) && (d.y > 0.01f) ) {
      n.x = -d.y;
      n.y = d.x;
      n.z = 0.0f; // a normal to the line direction
  } else {
      n.x = 1.0f;
      n.y = 0.0f;
      n.z = 0.0f;
  }

  LineVertexFormat draw_points[2];
  draw_points[0].m_point.x = start.x;
  draw_points[0].m_point.y = start.y;
  draw_points[0].m_point.z = start.z;
  draw_points[0].m_color = m_Color1.ARGB();
  draw_points[0].m_u0 = 0.0f;
  draw_points[0].m_v0 = 0.0f;

  draw_points[1].m_point.x = end.x;
  draw_points[1].m_point.y = end.y;
  draw_points[1].m_point.z = end.z;
  draw_points[1].m_u0 = 1.0f;
  draw_points[1].m_v0 = 1.0f;

  draw_points[0].m_normal = n;
  draw_points[1].m_normal = n;

  if (m_ShadeMode == smFlat)
    draw_points[1].m_color = m_Color1.ARGB();
  else
    draw_points[1].m_color = m_Color2.ARGB();

  pDevice->SetFVF(D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1);
  pDevice->DrawPrimitiveUP(D3DPT_LINELIST, 1, (CONST void *)(&draw_points), sizeof(LineVertexFormat));
  return true;
}


bool UWBD3D_DrawHelper::DrawCircle(vec3 center, float radius)
{
  //Direct3D likes to recieve arrays of structs with vertex information. For simple, unlit
  //drawing; a vertex and color at the vertex will suffice.
  struct CircleVertexFormat
  {
    D3DXVECTOR3 m_point;  // vertex location
    D3DXVECTOR3 m_normal;
    DWORD m_color;        // color at the vertex
    FLOAT m_u0, m_v0;     // stage 0 texture coordinates
  };

  D3DXVECTOR3 n(0.0f, 0.0f, 1.0f);

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if (radius <= 0.0f || NULL == pDevice)
    return false;

  // We are going to draw this circle with TriangleFan
  int kNumPts = 40;
  if(lodLow == m_lod)
    kNumPts = 10;
  else if(lodMedium == m_lod)
    kNumPts = 20;

  CircleVertexFormat* v = new CircleVertexFormat[kNumPts];
  
  float delta = (float)(UWB_ToRadians(360.0 / (kNumPts-2)));

  unsigned int flat_color = m_Color1.ARGB();

  v[0].m_point.x = center.x;
  v[0].m_point.y = center.y;
  v[0].m_point.z = center.z;
  v[0].m_color = flat_color;
  v[0].m_u0 = 0.5f;
  v[0].m_v0 = 0.5f;
  v[0].m_normal = n;

  for (int i=0; i<(kNumPts-2); i++)
  {
    v[i+1].m_point.x = center.x + (radius * cosf(i*delta));
    v[i+1].m_point.y = center.y + (radius * sinf(i*delta));
    v[i+1].m_normal = n;
    v[i+1].m_u0 = 0.5f + (0.5f * cosf(i*delta));
    v[i+1].m_v0 = 0.5f - (0.5f * sinf(i*delta));
    v[i+1].m_point.z = center.z;
    v[i+1].m_color = flat_color;
  }
  v[kNumPts-1] = v[1];

  if( smGouraud == m_ShadeMode )
    v[0].m_color = m_Color2.ARGB();

    pDevice->SetFVF(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1);
    pDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, kNumPts-2, (CONST void *)v, sizeof(CircleVertexFormat));

  delete v;
  return true;
}

bool UWBD3D_DrawHelper::EnableBlending( bool on )
{
  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(pDevice)
  {
    if( on )
    {
      pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
      pDevice->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_SRCALPHA);
      pDevice->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_INVSRCALPHA);
      pDevice->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
    }
    else
      pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
  }
  return true;
}

bool UWBD3D_DrawHelper::EnableTexture(bool on)
{
  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(pDevice)
  {
    if( on )
      UWBD3D_GraphicsSystem::GetSystem().ActivateTexture( m_TexFileName.c_str() );
    else
      UWBD3D_GraphicsSystem::GetSystem().DeactivateTexture();
  }
  return true;
}


void UWBD3D_DrawHelper::SetShadeMode(eShadeMode mode)
{
  __super::SetShadeMode(mode);

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(pDevice)
  {
    if( smGouraud == m_ShadeMode )
      pDevice->SetRenderState(D3DRS_SHADEMODE, D3DSHADE_GOURAUD);
    else
      pDevice->SetRenderState(D3DRS_SHADEMODE, D3DSHADE_FLAT);
  }
}

void UWBD3D_DrawHelper::SetFillMode(eFillMode mode)
{
  __super::SetFillMode(mode);

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(pDevice)
  {
    if( fmSolid == m_FillMode )
      pDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
    else if( fmWireframe == m_FillMode )
      pDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
    else
      pDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_POINT);
  }
}

bool UWBD3D_DrawHelper::DrawRectangleXY(vec3 c1, vec3 c2)
{
  //Direct3D likes to recieve arrays of structs with vertex information. For simple, unlit
  //drawing; a vertex and color at the vertex will suffice.
  struct VertexFormat
  {
    D3DXVECTOR3 m_point;  // vertex location
    D3DXVECTOR3 m_normal;
    DWORD m_color;        // color at the vertex
    FLOAT m_u0, m_v0;     // stage 0 texture coordinates
  };

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if (NULL == pDevice)
    return false;

  VertexFormat vf[6];
  unsigned int flat_color = m_Color1.ARGB();
  
  vec3 mid = 0.5 * (c1 + c2);
  vec3 min, max;
  min.x = c1.x < c2.x ? c1.x : c2.x;
  min.y = c1.y < c2.y ? c1.y : c2.y;
  max.x = c1.x >= c2.x ? c1.x : c2.x;
  max.y = c1.y >= c2.y ? c1.y : c2.y;
  min.z = max.z = 0;

  vf[0].m_point.x = mid.x;
  vf[0].m_point.y = mid.y;
  vf[0].m_u0 = 0.5f;
  vf[0].m_v0 = 0.5f;

  vf[1].m_point.x = min.x;
  vf[1].m_point.y = max.y;
  vf[1].m_u0 = 1.0f;
  vf[1].m_v0 = 0.0f;
  
  vf[2].m_point.x = min.x;
  vf[2].m_point.y = min.y;
  vf[2].m_u0 = 1.0f;
  vf[2].m_v0 = 1.0f;
  
  vf[3].m_point.x = max.x;
  vf[3].m_point.y = min.y;
  vf[3].m_u0 = 0.0f;
  vf[3].m_v0 = 1.0f;
  
  vf[4].m_point.x = max.x;
  vf[4].m_point.y = max.y;
  vf[4].m_u0 = 0.0f;
  vf[4].m_v0 = 0.0f;

  vf[5] = vf[1];

  for (int i=0; i<6; i++)
  {
    vf[i].m_point.z = 0;
    vf[i].m_color = flat_color;
    vf[i].m_normal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
  }

  if( smGouraud == m_ShadeMode )
    vf[0].m_color = m_Color2.ARGB();

  pDevice->SetFVF(D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1);
  pDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 4, (CONST void *)vf, sizeof(VertexFormat));

  return true;
}

bool UWBD3D_DrawHelper::DrawRectangleXZ(vec3 c1, vec3 c2)
{
  //Direct3D likes to recieve arrays of structs with vertex information. For simple, unlit
  //drawing; a vertex and color at the vertex will suffice.
  struct VertexFormat
  {
    D3DXVECTOR3 m_point;  // vertex location
    D3DXVECTOR3 m_normal;
    DWORD m_color;        // color at the vertex
    FLOAT m_u0, m_v0;     // stage 0 texture coordinates
  };

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if (NULL == pDevice)
    return false;

  VertexFormat vf[6];
  unsigned int flat_color = m_Color1.ARGB();
  
  vec3 mid = 0.5 * (c1 + c2);
  vec3 min, max;
  min.x = c1.x < c2.x ? c1.x : c2.x;
  min.z = c1.z < c2.z ? c1.z : c2.z;
  max.x = c1.x >= c2.x ? c1.x : c2.x;
  max.z = c1.z >= c2.z ? c1.z : c2.z;
  min.y = max.y = 0;

  vf[0].m_point.x = mid.x;
  vf[0].m_point.z = mid.z;
  vf[0].m_u0 = 0.5f;
  vf[0].m_v0 = 0.5f;

  vf[1].m_point.x = min.x;
  vf[1].m_point.z = max.z;
  vf[1].m_u0 = 1.0f;
  vf[1].m_v0 = 0.0f;
  
  vf[2].m_point.x = min.x;
  vf[2].m_point.z = min.z;
  vf[2].m_u0 = 1.0f;
  vf[2].m_v0 = 1.0f;
  
  vf[3].m_point.x = max.x;
  vf[3].m_point.z = min.z;
  vf[3].m_u0 = 0.0f;
  vf[3].m_v0 = 1.0f;
  
  vf[4].m_point.x = max.x;
  vf[4].m_point.z = max.z;
  vf[4].m_u0 = 0.0f;
  vf[4].m_v0 = 0.0f;

  vf[5] = vf[1];

  for (int i=0; i<6; i++)
  {
    vf[i].m_point.y = 0;
    vf[i].m_color = flat_color;
    vf[i].m_normal = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
  }

  if( smGouraud == m_ShadeMode )
    vf[0].m_color = m_Color2.ARGB();

  pDevice->SetFVF(D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1);
  pDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 4, (CONST void *)vf, sizeof(VertexFormat));

  return true;
}

bool UWBD3D_DrawHelper::InitializeModelTransform()
{
  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(NULL == pDevice)
    return false;
  if(NULL == m_pMatrixStack)
  {
    D3DXCreateMatrixStack(0 , &m_pMatrixStack);
    if(NULL == m_pMatrixStack)
      return false;
  }
  m_pMatrixStack->LoadIdentity();
  pDevice->SetTransform( D3DTS_WORLD, m_pMatrixStack->GetTop() );
  return true;
}

bool UWBD3D_DrawHelper::PushModelTransform()
{
  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(NULL == pDevice)
    return false;
  if(NULL == m_pMatrixStack)
    InitializeModelTransform();
  
  m_pMatrixStack->Push();
  pDevice->SetTransform( D3DTS_WORLD, m_pMatrixStack->GetTop() );
  return true;
}

bool UWBD3D_DrawHelper::PopModelTransform()
{
  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(NULL == pDevice)
    return false; 
  if(NULL == m_pMatrixStack)
     InitializeModelTransform();
  
  m_pMatrixStack->Pop();
  pDevice->SetTransform( D3DTS_WORLD, m_pMatrixStack->GetTop() );
  return true;
}


bool UWBD3D_DrawHelper::AccumulateModelTransform(vec3 translation,
                                          vec3 scale,
                                          mat3 rotation,
                                          vec3 pivot)
{
  if(NULL == m_pMatrixStack)
    InitializeModelTransform();

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(NULL == pDevice)
    return false;
  
  D3DXVECTOR3 xaxis(1,0,0);
  D3DXVECTOR3 yaxis(0,1,0);
  D3DXVECTOR3 zaxis(0,0,1);

  D3DXMATRIX m;
  m._11 = rotation[0][0]; m._12 = rotation[0][1]; m._13 = rotation[0][2];  m._14 = 0.0f;
  m._21 = rotation[1][0]; m._22 = rotation[1][1]; m._23 = rotation[1][2];  m._24 = 0.0f;
  m._31 = rotation[2][0]; m._32 = rotation[2][1]; m._33 = rotation[2][2];  m._34 = 0.0f;
  m._41 = 0.0f;           m._42 = 0.0f;           m._43 = 0.0f;            m._44 = 1.0f;
   
  m_pMatrixStack->TranslateLocal( translation.x, translation.y, translation.z);
  m_pMatrixStack->TranslateLocal(pivot.x, pivot.y, pivot.z);
    m_pMatrixStack->MultMatrixLocal(&m);
    m_pMatrixStack->ScaleLocal(scale.x, scale.y, scale.z);
  m_pMatrixStack->TranslateLocal(-pivot.x, -pivot.y, -pivot.z);

  pDevice->SetTransform( D3DTS_WORLD, m_pMatrixStack->GetTop() );

  return true;
}

bool UWBD3D_DrawHelper::TransformPoint(vec3& point)
{
  D3DXVECTOR3 pt;
  pt.x = point.x; pt.y = point.y; pt.z = point.z;
  D3DXVec3TransformCoord(&pt, &pt, m_pMatrixStack->GetTop() );
  point.x = pt.x;
  point.y = pt.y;
  point.z = pt.z;
  return true;
}

bool UWBD3D_DrawHelper::EnableLighting(bool on)
{
  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice(); 
  if (on)
  {
    D3DMATERIAL9 m;
    UWB_Color ambient, diffuse, emissive, specular;
    m_Material.GetAmbient(ambient);
    m_Material.GetSpecular(specular);
    m_Material.GetDiffuse(diffuse);
    m_Material.GetEmissive(emissive);
    m.Power = m_Material.GetPower();
        #define ConvertColor(dc, uc) {  \
          dc.r = uc.PercentRed();   \
          dc.g = uc.PercentGreen(); \
          dc.b = uc.PercentBlue();  \
          dc.a = uc.PercentAlpha(); \
      }
      ConvertColor(m.Ambient, ambient)
      ConvertColor(m.Specular, specular)
      ConvertColor(m.Diffuse, diffuse)
      ConvertColor(m.Emissive, emissive)
    
    pDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
    pDevice->SetMaterial( &m );
  }
  else
  {
    pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
  }
  return true;
}