#include "uwbgl_D3DPrimitiveMeshCustom1.h"
#include "uwbgl_D3DGraphicsSystem7.h"

const int UWBD3D_PrimitiveMeshCustom::kInvalidVertexIndex = -1;

UWBD3D_PrimitiveMeshCustom::UWBD3D_PrimitiveMeshCustom
(
	int		    					numControlVertices,
  const wchar_t      *outFileName
  ) : UWBD3D_PrimitiveMesh(outFileName)
    ,m_SelectedVertexIndex(kInvalidVertexIndex)
    ,m_Mesh(NULL)
    ,m_DrawNormal(false)
    ,m_DrawControlGrid(false)
{
	m_NumControlVertices = numControlVertices;
	m_NumVertices = m_NumControlVertices * m_NumControlVertices;
	m_NumFaces = (m_NumControlVertices-1) * (m_NumControlVertices-1) * 2;
	m_VertexDistance = 2.0f/float(m_NumControlVertices-1.0f);

  if (outFileName == NULL)
      outFileName = L"YourMeshFileName.x";
  m_resource_filename = outFileName;

  // at this point the graphics device may (or may not) have been created, we will
  // defer the actual initialization of the mesh till when we first draw.
  m_Initialized = false;
}

bool UWBD3D_PrimitiveMeshCustom::InitializeMesh() 
{
  UWBD3D_GraphicsSystem& graphics = UWBD3D_GraphicsSystem::GetSystem();
  LPDIRECT3DDEVICE9 device = graphics.GetD3DDevice();

  if (NULL == device)
    return false;
  
  //
	// Now create the mesh in D3D ...
	//
	D3DXCreateMeshFVF(m_NumFaces, m_NumVertices, D3DXMESH_SYSTEMMEM, DeviceVertexFormatCode, device, &m_Mesh);

	ComputeSurface();

  D3DXSaveMeshToX(m_resource_filename.c_str(), m_Mesh, NULL, NULL, NULL, 0, D3DXF_FILEFORMAT_TEXT);
  
  m_Mesh->Release();
  m_Mesh = NULL;

  // After we have saved the .x file, now load it in as a resource
  UWBD3D_MeshResource *pMesh = graphics.GetMeshResource( m_resource_filename.c_str() );
  if (pMesh) {
    m_Mesh = pMesh->GetD3DMesh();
    m_Initialized = true;
  }
  return m_Initialized;
}

void
UWBD3D_PrimitiveMeshCustom::ComputeSurface()
{
	int i;
	// 
	// Get/Lock/Load the VertexBuffer
	//
	void *pVBuffer;
	m_Mesh->LockVertexBuffer( 0, &pVBuffer );
			DeviceVertexFormat *v = (DeviceVertexFormat *) pVBuffer;
			//
			//
			// the mesh geometry is defined in the range of ( -1 <->  1) in the x-z plane.
			// the vertices of the mesh are uniformlly distributed in this range.
			//
			int   nthRow = 0, nthColumn = 0;
	
			for (i=0; i<m_NumVertices; i++) {
					v[i].m_Point.x = -1.0f + (nthColumn * m_VertexDistance);
					v[i].m_Point.y = 0.0f;
					v[i].m_Point.z =  1.0f - (nthRow * m_VertexDistance);
					nthColumn++;
					if (nthColumn>=m_NumControlVertices) {
						nthRow++;
						nthColumn = 0;
					}
	
					v[i].m_Normal.x = 0.0f;	// ComputeVertexNormal will do this right.
					v[i].m_Normal.y = 1.0f;
					v[i].m_Normal.z = 0.0f;
	
					v[i].m_U = (v[i].m_Point.x / 2.0f) + 0.5f;
					v[i].m_V = (v[i].m_Point.z / 2.0f) + 0.5f;
			}
			m_Mesh->UnlockVertexBuffer();

			//
			// now fill in the strips of triangle (face) index
			// 
			// each strip has two rows of triangle
			//
			// Here are the observations:
			//
			// on nth strip: 
			//		lower row has index: 
			//				nth*m_NumControlVertices, (nth*m_NumControlVertices)+1, (nth+1)*m_NumControlVertices
			//
			//      upper row has inex:
			//				(nth+1)*m_NumControlVertices, (nth*m_NumControlVertices)+1, ((nth+1)*m_NumControlVertices)+1
			//
			// And each row have m_NumControlVertices-1 number of triangles ...
			//
			int indexCount=0;

			//
			// 2. Get/Lock/Load the IndexBuffer
			void *pIBuffer;
			m_Mesh->LockIndexBuffer( 0, &pIBuffer);
			WORD *indexBuffer = (WORD *)pIBuffer;
					for (int strip = 0; strip < m_NumControlVertices-1; strip++) {
							int lowV0 = strip*m_NumControlVertices;
							int lowV1 = lowV0 + 1;
							int lowV2 = (strip+1)*m_NumControlVertices;
	
							int hiV0 = lowV0 + m_NumControlVertices;
							int hiV1 = lowV1;
							int hiV2 = hiV0 + 1;
	
							for (int rowTri=0; rowTri < m_NumControlVertices-1; rowTri++) {
									//
									// lower row ...
									indexBuffer[indexCount++] = lowV0++;
									indexBuffer[indexCount++] = lowV1++;
									indexBuffer[indexCount++] = lowV2++;
	
									//
									// upper row
									indexBuffer[indexCount++] = hiV0++;
									indexBuffer[indexCount++] = hiV1++;
									indexBuffer[indexCount++] = hiV2++;
							}
					}
			m_Mesh->UnlockIndexBuffer();

		//
		// Get Help to compute Normal ...
		//		Do this before redraw ...
		//		D3DXComputeNormals(m_Mesh, NULL);
}

UWBD3D_PrimitiveMeshCustom::~UWBD3D_PrimitiveMeshCustom()
{
	m_Mesh = NULL;
}
